/*
    FEARLUS model0-6-2: LandParcel.h
    Copyright (C) 1999-2002  Macaulay Institute
    Copyright (C) 1999-2002  Macaulay Institute
    Copyright (C) 1999-2002  Macaulay Institute

    This file is part of FEARLUS model0-6-2, an agent-based model of land use
    change.

    FEARLUS model0-6-2 is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    FEARLUS model0-6-2 is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details. (LICENCE file in
    this directory.)

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    Contact information:
      Gary Polhill,
      Macaulay Institute, Craigiebuckler, Aberdeen, AB15 8QH. United Kingdom
      g.polhill@macaulay.ac.uk
*/
/*


Interface for the LandParcel class. This stores a bit string, an
environment, some location data for the environment, a yield, a yield
history, a current land use, a land use history, a land manager, and a
land manager history.



*/

#import <objectbase/SwarmObject.h>
#import <gui.h>
#import <stdio.h>
#import "Strategy.h"
#import "Spatial.h"

@class LandManager, LandUse, Parameter, BitString, Environment, Tube,
  Coordinate2DList;

@interface LandParcel: SwarmObject <Grid2DSpatial> {
  // Instance variables with a causal influence
  Parameter *parameter;
  BitString *attributes;
  Environment *environment;
  int x, y;
  BOOL newLandManager;
  unsigned yield;
  LandManager *landManager;
  LandUse *landUse;
  Tube *yieldMemory, *landUseMemory;
  // Instance variables for internal machinations
  LandManager *nextLandManager;
  LandUse *nextLandUse;
  // Instance variables for observation purposes
  id landParcelHistory;
				// Actually, this isn't just for observation
				// purposes, as the land managers interrogate
                                // this for the yield history.
  int maxGraphColour;		// Maximum colour available for graphing.
				// (This will be a grey level thing)
  unsigned nLUChange;	   	// Number of changes of land use
  unsigned nLMgrChange;		// Number of changes of land manager
  double changeLURank;		// Rank in environment of change in land use
  double changeLMgrRank;       	// Rank in environment of change in land mgr
  unsigned northNeighbourMatch;	// Degree of match between attributes of self
                                // and neighbour to the north
  unsigned westNeighbourMatch;	// As per northNeighbourMatch, but for west.
  id <Strategy> strategy;	// The method used to determine the land use
  unsigned pin;
  BOOL displayMgrNbr;
  BOOL displayPhysNbr;
  int lpBitToDisplay;
}

+create: (id)z;
-(unsigned)getPIN;
-(void)setParameters: (Parameter *)p;
-(void)setNextLandUseTypeTo: (LandUse *)lu;
-(void)setNextLandManagerTo: (LandManager *)lm;
-(LandManager *)getLandManager;
-(void)setLandManager: (LandManager *)lm;
-(LandUse *)getLandUse;
-(LandUse *)recallLandUseAgo: (unsigned)yrs;
-(unsigned)recallYieldAgo: (unsigned)yrs;
-(void)toggleBits: (BitString *)bs;
-(void)setBitLength: (unsigned)bl;
-(void)randomiseBits;
-(void)printHistoryToFile: (FILE *)fp;
-(void)updateLandUse;
-(void)updateLandManager;
-(void)recordHistory;
-(void)setLandUseTypeTo: (LandUse *)lu;
				// This method should only be used by the
				// land managers to set the initial land use
-(void)calculateYield;
-(void)setXPos: (int)xx YPos: (int)yy;
-(void)setEnvironment: (Environment *)env;
-(Environment *)getEnvironment;
-(void)setStrategy: (id <Strategy>)s;
-(id <Strategy>)getStrategy;
-(void)setMaxGraphColour: (int)col;
-(BitString *)getAttributes;
-(int)getOneAttribute: (unsigned)a;
-(void)setAttribute: (unsigned)a toValue: (int)v;
-(int)getX;
-(int)getY;
-(LandParcel *)getNextNeighbour;
-(unsigned)getYield;
-(int)getIntYield;
-(unsigned)getLastYield;
-(int)yieldDown;
-(void)setChangeLURank: (double)rank;
-(void)setChangeLMgrRank: (double)rank;
-(unsigned)getNLUChange;
-(unsigned)getNLMgrChange;
-(BOOL)hasANewLandManager;
-(void)determineNWNeighbourAttributeMatch;
-(void)toggleMgrNbrDisplay;
-(void)togglePhysNbrDisplay;
-drawPhysNbrsOf: (LandParcel *)lp
	     on: (id <Raster>)aRaster
	 memory: (Coordinate2DList *)clist;
-drawSelfOn: (id <Raster>)aRaster;
-drawMatchLinesOn: (id <Raster>)aRaster;
-drawNbrLinesOn: (id <Raster>)aRaster;
-drawNLUChangeOn: (id <Raster>)aRaster;
-drawNMgrChangeOn: (id <Raster>)aRaster;
-drawMgrOn: (id <Raster>)aRaster;
-drawSubPopOn: (id <Raster>)aRaster;
-drawMgrBoundariesOn: (id <Raster>)aRaster;
-drawChangeLURankOn: (id <Raster>)aRaster;
-drawChangeLMgrRankOn: (id <Raster>)aRaster;
-drawStrategyOn: (id <Raster>)aRaster;
-drawBitStringOn: (id <Raster>)aRaster;
-setBitToDisplay: (unsigned)bit;
-incBitToDisplay;
-decBitToDisplay;
-(void)drop;

@end
